local Skada = Skada
--local L = LibStub("AceLocale-3.0"):GetLocale("Skada", false)

local mod = Skada:NewModule("Stagger")
local modDetails = Skada:NewModule("Stagger details")

local playerData = {}

local function printdebug(...)
      --print(...)
end

local function getSetDuration(set)
      if set.time > 0 then
            return set.time
      else
            local endtime = set.endtime
            if not endtime then
                  endtime = time()
            end
            return endtime - set.starttime
      end
end

local function nextDataset(win, context)
      context.index = context.index or 1

      local dataset = win.dataset[context.index] or {}
      dataset.id = context.index
      win.dataset[context.index] = dataset

      context.index = context.index + 1
      return dataset
end

local function getPurifyPercent(unitGUID)
      local talentGroup = GetActiveSpecGroup()
      _, _, _, elusiveDanceSelected, _ = GetTalentInfo(7, 1, talentGroup, true, unitGUID)

      local purifyPercent = 0.5
      if elusiveDanceSelected then
            purifyPercent = purifyPercent + 0.15
      end
      printdebug("Purify percent for "..unitGUID..": "..purifyPercent)
      return purifyPercent
end

local tick = {}
local function logStaggerTick(set, tick)
      local player = Skada:get_player(set, tick.dstGUID, tick.dstName)
      if player then
            player.stagger.taken = player.stagger.taken + tick.samount
            player.stagger.tickCount = player.stagger.tickCount + 1
            if player.stagger.tickMax < tick.samount then
                  player.stagger.tickMax = tick.samount
            end
            if tick.timeSinceLast then
                  player.stagger.duration = player.stagger.duration + tick.timeSinceLast
                  if tick.timeSinceLast > 2 then
                        player.stagger.freezeDuration = player.stagger.freezeDuration + (tick.timeSinceLast - 0.5)
                  end
            end
      end
end

local purify = {}
local function logStaggerPurify(set, purify)
      local player = Skada:get_player(set, purify.srcGUID, purify.srcName)
      if player then
            player.stagger.purified = player.stagger.purified + purify.samount
            player.stagger.purifyCount = player.stagger.purifyCount + 1
            if player.stagger.purifyMax < purify.samount then
                  player.stagger.purifyMax = purify.samount
            end
      end
end

local function SpellDamage(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
      local spellId, spellName, spellSchool, samount, soverkill, sschool, sresisted, sblocked, sabsorbed, scritical, sglancing, scrushing, soffhand, _ = ...
      if dstGUID == UnitGUID("player") then
            if spellId == 124255 then
                  local playerData = playerData[dstGUID]

                  tick.dstGUID = dstGUID
                  tick.dstName = dstName
                  tick.samount = samount
                  if playerData.lastTickTime then
                        tick.timeSinceLast = timestamp - playerData.lastTickTime
                  end
                  logStaggerTick(Skada.current, tick)
                  logStaggerTick(Skada.total, tick)

                  if UnitStagger(dstName) > 0 then
                        playerData.lastTickTime = timestamp
                  else
                        playerData.lastTickTime = nil
                  end
            end
      end
end

local function SpellCast(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
      local spellId, spellName, spellSchool = ...
      if spellId == 119582 then
            local purifiedAmount = UnitStagger(srcName) * getPurifyPercent(srcGUID)
            printdebug(srcName.." purified "..purifiedAmount.." stagger")

            purify.srcGUID = srcGUID
            purify.srcName = srcName
            purify.samount = purifiedAmount
            logStaggerPurify(Skada.current, purify)
            logStaggerPurify(Skada.total, purify)
      end
end

function mod:OnEnable()
      mod.metadata = {showspots = false, click1 = modDetails}
      modDetails.metadata = {showspots = false, ordersort = true}
      
      Skada:RegisterForCL(SpellDamage, 'SPELL_PERIODIC_DAMAGE', {src_is_interesting = true, dst_is_interesting_nopets = false})
      Skada:RegisterForCL(SpellCast, 'SPELL_CAST_SUCCESS', {src_is_interesting = true, dst_is_interesting_nopets = false})

      Skada:AddMode(self, "Stagger")
end

function mod:OnDisable()
      Skada:RemoveMode(self)
end

function modDetails:Enter(win, id, label)
      modDetails.playerid = id
      modDetails.title = label.."'s Stagger"
end

function modDetails:Update(win, set)
      local player = Skada:find_player(set, self.playerid)
      if player then
            local playerStagger = player.stagger

            local damageStaggered = playerStagger.taken + playerStagger.purified
            if damageStaggered == 0 then
                  return
            end
            local setDuration = getSetDuration(set)

            local datasetContext = {}

            local staggered = nextDataset(win, datasetContext)
            staggered.label = "Damage Staggered"
            staggered.valuetext = Skada:FormatNumber(damageStaggered)
            staggered.value = 1

            local staggerTaken = nextDataset(win, datasetContext)
            staggerTaken.label = "Taken"
            staggerTaken.valuetext = Skada:FormatNumber(playerStagger.taken)..(" (%02.1f%%)"):format(playerStagger.taken / damageStaggered * 100)
            staggerTaken.value = playerStagger.taken / damageStaggered

            if playerStagger.purifyCount > 0 then
                  local staggerPurified = nextDataset(win, datasetContext)
                  staggerPurified.label = "Purified"
                  staggerPurified.valuetext = Skada:FormatNumber(playerStagger.purified)..(" (%02.1f%%)"):format(playerStagger.purified / damageStaggered * 100)
                  staggerPurified.value = playerStagger.purified / damageStaggered

                  local staggerPurifiedAvg = nextDataset(win, datasetContext)
                  staggerPurifiedAvg.label = "Purified (average)"
                  staggerPurifiedAvg.valuetext = Skada:FormatNumber(playerStagger.purified / playerStagger.purifyCount).." ("..playerStagger.purifyCount.."x)"
                  staggerPurifiedAvg.value = (playerStagger.purified / playerStagger.purifyCount) / damageStaggered
            end

            if setDuration > 0 then
                  local staggerDuration = nextDataset(win, datasetContext)
                  staggerDuration.label = "Stagger Duration"
                  staggerDuration.valuetext = ("%02.1fs"):format(playerStagger.duration)
                  staggerDuration.value = playerStagger.duration / setDuration
                  
                  if playerStagger.freezeDuration > 2 then
                        local freezeDuration = nextDataset(win, datasetContext)
                        freezeDuration.label = "Freeze Duration"
                        freezeDuration.valuetext = ("%02.1fs"):format(playerStagger.freezeDuration)..(" (%02.1f%%)"):format(playerStagger.freezeDuration / playerStagger.duration * 100)
                        freezeDuration.value = playerStagger.freezeDuration / setDuration
                  end
            end

            local tickMax = nextDataset(win, datasetContext)
            tickMax.label = "Tick (max)"
            tickMax.valuetext = Skada:FormatNumber(playerStagger.tickMax)
            tickMax.value = playerStagger.tickMax / damageStaggered

            win.metadata.maxvalue = 1
      end
end

function mod:AddPlayerAttributes(player, set)
      if not player.stagger then
            player.stagger = 
            {
                  taken = 0,
                  purified = 0,
                  purifyCount = 0,
                  purifyMax = 0,

                  tickMax = 0,
                  tickCount = 0,

                  duration = 0,
                  freezeDuration = 0,
            }
      end
      if not playerData[player.id] then
            playerData[player.id] = {}
      end            
end

function mod:GetSetSummary(set)
      local totalPurified = 0
      for i, player in ipairs(set.players) do
            if player.stagger then
                  totalPurified = totalPurified + player.stagger.purified
            end
      end
      return "(purified) "..Skada:FormatNumber(totalPurified)
end

function mod:Update(win, set)
      local nr = 1
      local max = 0

      for i, player in ipairs(set.players) do
            if player.stagger then

                  local d = win.dataset[nr] or {}
                  win.dataset[nr] = d

                  local value = player.stagger.taken + player.stagger.purified

                  d.id = player.id
                  d.label = player.name
                  d.value = value
                  d.valuetext = Skada:FormatNumber(value)
                  d.class = player.class
                  d.role = player.role

                  if max < value then
                        max = value
                  end

                  nr = nr + 1
            end
      end
      win.metadata.maxvalue = max
end
